###############   PSQF 7375 Advanced Longitudinal Example 2 using R   #############

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

#if (!require("psych")) install.packages("psych")
#library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

#if (!require("multcomp")) install.packages("multcomp")
#library(multcomp) # To do linear combinations in gls

#if (!require("emmeans")) install.packages("emmeans")
#library(emmeans) # To get model-implied means

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC in lmer

#if (!require("prediction")) install.packages("prediction")
#library(prediction) # To get predicted values like Stata does

###############################################################################
# Jonathan's function to add level-two means to values
addUnitMeans = function(data, unitVariable, meanVariables, newNames){
  # create unit-level data set:
  unitMeans = t(sapply(
    X = unique(data[, unitVariable]),
    FUN = function(x, data) {
      return(c(
        x,
        length(which(data[, unitVariable] == x)),
        apply(
          X = as.data.frame(data[which(data[, unitVariable] == x), meanVariables]),
          MARGIN = 2,
          FUN = mean,
          rm.na = TRUE
        )
      ))
    },
    data = data
  ))
  unitMeans = data.frame(unitMeans)
  names(unitMeans) = c(unitVariable, paste0("Nper", unitVariable), newNames)
  newData = merge(x = data, y = unitMeans, by = unitVariable)
  return(newData)
}


################################################################################
#####           BEGIN DATA MANIPULATION OF EXAMPLE 6 MODELS                #####
#####            CHANGE "filesave" TO YOUR DIRECTORY                       #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/23_PSQF7375_AdvLong/PSQF7375_AdvLong_Example6/"
filename = "AdvLong_Example6.xlsx"
setwd(dir=filesave)

# Import Example 6 stacked data
Example6 = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example6 = as.data.frame(Example6)
# Sort by PairID, TwinID, and Wave
sort_asc(data=Example6, PairID, TwinID, Wave) 

# Create pair-level (between-family) age at wave1
# Uses function from above to add pair means (to same data here)
Example6 = addUnitMeans(data=Example6, unitVariable="PairID",
                        meanVariables=c("agew1"), newNames=c("PairAge"))

# Center pair-level (between-family) age at 85 for use at level 3
Example6$PairAge85 = Example6$PairAge - 85
# Within-person center age for use at level-1 (VARIABLE CENTERING)
Example6$time = Example6$age - Example6$agew1
Example6$timesq = Example6$time * Example6$time
# Center zygosity so 0=MZ, 1=DZ, will be treated as numeric
Example6$IsDZ = Example6$zygosity - 1

############## Labels as comments only #########################
#BFage85 = "BFage85: Age at Wave1 (0=85)
#time =    "time: Time Since Wave1 (0= Age at Wave1)"
#timesq =  "timesq: Squared Time Since Wave1 (0= Age at Wave1)"
#IsDZ =    "IsDZ: Zygosity (0=MZ, 1=DZ)
#################################################################

# Select only cases with complete data per occasion
Example6 = Example6[complete.cases(Example6[ , c("info","time","PairAge","IsDZ")]),]

# Remove last occasion that is mostly missing data
Example6 = subset(x=Example6, Example6$Wave<5)

# Open external file to save results to
txtStart(file=paste0(filesave,"AdvLong_Example6_R_Output.txt"))


###################################################################################
#######        PART 1: VARIANCE PARTITIONING AND SATURATED MEANS            #######
###################################################################################

print("R Model 1a: Empty Means, Two-Level Model for Cognition Outcome")
Model1a = lmer(data=Example6, REML=TRUE, 
               formula=info~1+(1|Case))  # Level2+3 (Case = Unique PersonID)
llikAIC(Model1a); summary(Model1a); icc(Model1a)
print("LRT for removing random intercept"); ranova(Model1a, reduce.term=TRUE) 

txtComment(" ") # insert blank space in output

##################################################################################
print("R Model 1b: Empty Means, Three-Level Model for Cognition Outcome")
Model1b = lmer(data=Example6, REML=TRUE, 
               formula=info~1+(1|PairID)+(1|PairID:TwinID)) # L3 Pairs + L2 Twins
llikAIC(Model1b); summary(Model1b); icc(Model1b)
print("LRT for removing random intercepts"); ranova(Model1b, reduce.term=TRUE) 

txtComment(" ") # insert blank space in output

##################################################################################
print("R Age Model: Empty Means, Three-Level Model for Age Predictor")
EmptyAge = lmer(data=Example6, REML=TRUE, 
                formula=age~1+(1|PairID)+(1|PairID:TwinID)) # L3 Pairs + L2 Twins
llikAIC(EmptyAge); summary(EmptyAge); icc(EmptyAge)

txtComment(" ") # insert blank space in output

##################################################################################
print("R Model 1c: Saturated Wave Means, Three-Level Model for Cognition Outcome")
SatWave = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~0+as.factor(Wave)+(1|PairID)+(1|PairID:TwinID)) 
summary(SatWave)

# Save and convert list output of means into data frame, compute CIs
SumSatWave = summary(SatWave)
WaveMeans = data.frame(SumSatWave$coefficients)
WaveMeans$LCI = WaveMeans$Estimate-1.96*WaveMeans$Std..Error
WaveMeans$UCI = WaveMeans$Estimate+1.96*WaveMeans$Std..Error
# Extract Wave from row label from character 20-21
WaveMeans$Wave = as.numeric(substr(x=rownames(WaveMeans), start=16, stop=17))
WaveMeans # Show result in output

# Save plot: open file, make plot, close file
png(file="R Saturated Means by Wave.png")  # open file
plot(x=WaveMeans$Wave, y=WaveMeans$Estimate, type="l",
     ylim=c(min(WaveMeans$LCI), max(WaveMeans$UCI)),
     xlab="Rounded Wave in Years", ylab="Predicted Info")
# Loop over points to add error bars
estimate=1
for (estimate in 1:nrow(WaveMeans)){
  points(x =   WaveMeans$Wave[estimate], y=WaveMeans$Estimate[estimate])
   lines(x = c(WaveMeans$Wave[estimate], WaveMeans$Wave[estimate]), 
         y = c(WaveMeans$LCI[estimate], WaveMeans$UCI[estimate]))
}; dev.off()  # close file

txtComment(" ") # insert blank space in output


###################################################################################
#######            PART 2: FIXED AND RANDOM EFFECTS OF TIME                 #######
###################################################################################

print("R Model 2a: Fixed Quadratic, Random Intercepts for L2 Twin and L3 Pair")
Model2a = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~1+time+I(time^2)+(1|PairID)+(1|PairID:TwinID))
llikAIC(Model2a); summary(Model2a)

txtComment(" ") # insert blank space in output

###################################################################################
print("R Model 2b: Add Random Linear Time over L2 Twins")
Model2b = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~1+time+I(time^2)+(1|PairID)+(1+time|PairID:TwinID))
llikAIC(Model2b); summary(Model2b)
print("Test random linear time over L2 twins"); ranova(Model2b, reduce.term=TRUE)

txtComment(" ") # insert blank space in output

###################################################################################
print("R Model 2c: Add Random Linear Time over L3 Pairs")
Model2c = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~1+time+I(time^2)+(1+time|PairID)+(1+time|PairID:TwinID))
llikAIC(Model2c); summary(Model2c)
print("Test random linear time over L3 pairs"); ranova(Model2c, reduce.term=TRUE)

txtComment(" ") # insert blank space in output

###################################################################################
#print("R Model 2d: Add Random Quadratic Time over L2 Twins -- won't even try to run")
#Model2d = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
#               formula=info~1+time+I(time^2)+(1+time|PairID)+(1+time+timesq|PairID:TwinID))
#llikAIC(Model2d); summary(Model2d)
#print("Test random quadratic time over L2 twins"); ranova(Model2d, reduce.term=TRUE)

#txtComment(" ") # insert blank space in output

###################################################################################
#print("R Model 2e: Add Random Quadratic Time over L3 Pairs -- won't event try to run")
#Model2e = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
#               formula=info~1+time+I(time^2)+(1+time+timesq|PairID)+(1+time+timesq|PairID:TwinID))
#llikAIC(Model2e); summary(Model2e)
#print("Test random quadratic time over L3 pairs"); ranova(Model2e, reduce.term=TRUE)

#txtComment(" ") # insert blank space in output

###################################################################################
print("R Model 3a: Add Baseline Age as Predictor of Pair Intercepts and Change")
Model3a = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~1+time+I(time^2)+PairAge85 +time:PairAge85 +I(time^2):PairAge85
                            +(1+time|PairID)+(1+time|PairID:TwinID))
llikAIC(Model3a); summary(Model3a)
print("Trajectory diffs by age"); contestMD(Model3a, ddf="Satterthwaite", 
      L=rbind(c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output

###################################################################################
print("R Model 3b: Add Zygosity as Predictor of Pair Intercepts and Change")
Model3b = lmer(data=Example6, REML=TRUE,      # L3 Pairs + L2 Twins
               formula=info~1+time+I(time^2)+PairAge85+IsDZ 
               +time:PairAge85 +I(time^2):PairAge85 +time:IsDZ +I(time^2):IsDZ 
               +PairAge85:IsDZ +time:PairAge85:IsDZ +I(time^2):PairAge85:IsDZ
                +(1+time|PairID)+(1+time|PairID:TwinID))
llikAIC(Model3b); summary(Model3b)
print("Trajectory diffs by zygosity"); contestMD(Model3b, ddf="Satterthwaite", 
      L=rbind(c(0,0,0,0,1,0,0,0,0,0,0,0),c(0,0,0,0,0,0,0,1,0,0,0,0),
              c(0,0,0,0,0,0,0,0,1,0,0,0),c(0,0,0,0,0,0,0,0,0,1,0,0),
              c(0,0,0,0,0,0,0,0,0,0,1,0),c(0,0,0,0,0,0,0,0,0,0,0,1)))

txtComment(" ") # insert blank space in output

######################################################################################
#print("R Model 3b: Add Zygosity as Predictor of Pair Intercepts and Change using LME")
#print("Would be needed to compare next heterogeneous variance version against")
#Model3b = lme(data=Example6, method="REML",      # L3 Pairs + L2 Twins
#              fixed=info~1+time+I(time^2)+PairAge85+IsDZ 
#               +time:PairAge85 +I(time^2):PairAge85 +time:IsDZ +I(time^2):IsDZ 
#               +PairAge85:IsDZ +time:PairAge85:IsDZ +I(time^2):PairAge85:IsDZ,
#               random=list(PairID=~time,Case=~time))
#-2*logLik(Model3b); summary(Model3b)

#txtComment(" ") # insert blank space in output

###################################################################################
#print("R Model 3c: Add Add Heterogeneous G and R matrices by Zygosity using LME")
#print("This is my best attempt, but it won't converge, so I don't know if it is right")
#Model3c = lme(data=Example6, method="REML",      # L3 Pairs + L2 Twins
#              fixed=info~1+time+I(time^2)+PairAge85+IsDZ 
#              +time:PairAge85 +I(time^2):PairAge85 +time:IsDZ +I(time^2):IsDZ 
#              +PairAge85:IsDZ +time:PairAge85:IsDZ +I(time^2):PairAge85:IsDZ,
#              random=list(PairID=pdSymm(~time*IsDZ),Case=pdSymm(~time*IsDZ)))
#-2*logLik(Model3c); summary(Model3c)

#txtComment(" ") # insert blank space in output

###########################################################################################
# Close external file of saved results
txtStop()



