// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

*********************************************************************************
******     BEGIN DATA MANIPULATION FOR EXAMPLE 6 THREE-LEVEL MODELS       *******
******               CHANGE "filesave" to your directory                  *******
*********************************************************************************

// Defining working directory for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
cd "C:\Dropbox\23_PSQF7375_AdvLong\PSQF7375_AdvLong_Example6"

// Import Example6 long data
clear // clear memory in case a dataset is already open
import excel "AdvLong_Example6.xlsx", firstrow case(preserve) clear 
// Sort by PairID TwinID Wave
sort PairID TwinID Wave
 
// Create pair-level (between-family) age at wave1
egen PairAge = mean(agew1), by(PairID) 
// Center pair-level (between-family) age at 85 for use at level 3
gen PairAge85 = PairAge - 85
label variable PairAge85 "PairAge85: Pair Mean Age at Wave1 (0=85)"
// Within-person center age for use at level-1 (VARIABLE CENTERING)
gen time = age - agew1
label variable time "time: Time since Wave1 (0= Age at Wave1)"
// Quadratic time to use as random slope
gen timesq = time*time
label variable timesq "timesq: Squared Time since Wave1 (0= Age at Wave1)"

// Center zygosity so 0=MZ, 1=DZ, will be treated as numeric
gen IsDZ = zygosity-1
label variable IsDZ "IsDV: Zygosity (0=MZ, 1=DZ)"

// Select only cases with complete data per occasion
egen nummiss = rowmiss(info time PairAge IsDZ)
drop if nummiss>0

// Remove last occasion that is mostly missing data
drop if Wave==5

// Open external file to save results to
log using AdvLong_Example6_STATA_Output.log, replace


******************************************************************************
******       PART 1: VARIANCE PARTITIONING AND SATURATED MEANS          ******
******************************************************************************

display "STATA Model 1a: Empty Means, Two-Level Model for Cognition Outcome"
mixed info ,  || Case:  ,    /// Level2+3 (Case = unique person ID)
              reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2    // Print -2LL for model
estat icc                     // Intraclass correlation 
estimates store TwoLevel      // Save LL for LRT


display "STATA Model 1b: Empty Means, Three-Level Model for Cognition Outcome"
mixed info ,  || PairID: ,   ///
              || TwinID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog 
display "-2LL = " e(ll)*-2    // Print -2LL for model
estat icc                     // Intraclass correlations 
estimates store ThreeLevel    // Save LL for LRT
lrtest ThreeLevel TwoLevel    // Compare three-level empty to two-level empty
     

display "STATA Age Model: Empty Means, Three-Level Model for Age Predictor"
mixed age ,  || PairID: ,    /// Level 3
             || TwinID: ,    /// Level 2
                reml dfmethod(satterthwaite) dftable(pvalue) nolog 
display "-2LL = " e(ll)*-2    // Print -2LL for model
estat icc                     // Intraclass correlations 
     

display "STATA Model 1c: Saturated Wave Means, Three-Level Model for Cognition Outcome"
mixed info i.Wave,  || PairID: ,    /// Level 3
                    || TwinID: ,    /// Level 2
            reml dfmethod(satterthwaite) dftable(pvalue) nolog 
margins i.Wave                         // Print saturated means by wave
marginsplot, name(sat_means, replace)  // Plot  saturated means by wave
graph export "STATA Saturated Means by Wave.png", replace
      
      
******************************************************************************
******          PART 2: FIXED AND RANDOM EFFECTS OF TIME                ******
******************************************************************************
     
display "STATA Model 2a: Fixed Quadratic, Random Intercepts for L2 Twin and L3 Pair"
mixed info c.time c.time#c.time ,  ///
           || PairID: ,            /// Level 3
           || TwinID: ,            /// Level 2 
              reml dfmethod(satterthwaite) dftable(pvalue) nolog 
display "-2LL = " e(ll)*-2          // Print -2LL for model
estimates store RI2RI3              // Save LL for LRT


display "STATA Model 2b: Add Random Linear Time over L2 Twins"
mixed info c.time c.time#c.time ,                ///
      || PairID:     ,                           /// Level 3
      || TwinID: time, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog
estat recovariance, relevel(TwinID) correlation  // L2 GCORR matrix
display "-2LL = " e(ll)*-2   // Print -2LL for model
estimates store RL2RI3       // Save LL for LRT
lrtest RL2RI3 RI2RI3         // Test random linear time over L2 twins


display "STATA Model 2c: Add Random Linear Time over L3 Pairs"
mixed info c.time c.time#c.time ,                ///
      || PairID: time, covariance(unstructured)  /// Level 3
      || TwinID: time, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog 
estat recovariance, relevel(PairID) correlation   // L3 GCORR matrix
estat recovariance, relevel(TwinID) correlation   // L2 GCORR matrix
display "-2LL = " e(ll)*-2   // Print -2LL for model
estimates store RL2RL3       // Save LL for LRT
lrtest RL2RL3 RL2RI3         // Test random linear time over L3 pairs


display "STATA Model 2d: Add Random Quadratic Time over L2 Twins"
mixed info c.time c.time#c.time ,                       ///
      || PairID: time       , covariance(unstructured)  /// Level 3
      || TwinID: time timesq, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog 
estat recovariance, relevel(PairID) correlation   // L3 GCORR matrix
estat recovariance, relevel(TwinID) correlation   // L2 GCORR matrix
display "-2LL = " e(ll)*-2   // Print -2LL for model
estimates store RQ2RL3       // Save LL for LRT
lrtest RQ2RL3 RL2RL3         // Test random quadratic time over L2 twins

/*
display "STATA Model 2e: Add Random Quadratic Time over L3 Pairs -- won't converge"
mixed info c.time c.time#c.time ,                       ///
      || PairID: time timesq, covariance(unstructured)  /// Level 3
      || TwinID: time timesq, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog 
estat recovariance, relevel(PairID) correlation   // L3 GCORR matrix
estat recovariance, relevel(TwinID) correlation   // L2 GCORR matrix
display "-2LL = " e(ll)*-2   // Print -2LL for model
estimates store RQ2RL3       // Save LL for LRT
lrtest RQ2RQ3 RQ2RL3         // Test random quadratic time over L3 pairs
*/     

display "STATA Model 3a: Add Baseline Age as Predictor of Pair Intercept and Change"
mixed info c.time c.time#c.time c.PairAge85 c.time#c.PairAge85 c.time#c.time#c.PairAge85, ///
      || PairID: time, covariance(unstructured)  /// Level 3
      || TwinID: time, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog 
estat recovariance, relevel(PairID) correlation   // L3 GCORR matrix
estat recovariance, relevel(TwinID) correlation   // L2 GCORR matrix
display "-2LL = " e(ll)*-2   // Print -2LL for model
// Trajectory diffs by age
test (c.PairAge85=0)(c.time#c.PairAge85=0)(c.time#c.time#c.PairAge85=0), small  
     

display "STATA Model 3b: Add Zygosity as Predictor of Pair Intercept and Change"
mixed info c.time c.time#c.time c.PairAge85 c.time#c.PairAge85 c.time#c.time#c.PairAge85  ///
           c.IsDZ c.time#c.IsDZ c.time#c.time#c.IsDZ c.PairAge85#c.IsDZ  ///
           c.time#c.PairAge85#c.IsDZ c.time#c.time#c.PairAge85#c.IsDZ,   ///
      || PairID: time, covariance(unstructured)  /// Level 3
      || TwinID: time, covariance(unstructured)  /// Level 2
         reml dfmethod(satterthwaite) dftable(pvalue) nolog 
estat recovariance, relevel(PairID) correlation   // L3 GCORR matrix
estat recovariance, relevel(TwinID) correlation   // L2 GCORR matrix
// Trajectory diffs by zygosity
test (c.IsDZ=0)(c.time#c.IsDZ=0)(c.time#c.time#c.IsDZ=0)(c.PairAge85#c.IsDZ=0) ///
     (c.time#c.PairAge85#c.IsDZ=0)(c.time#c.time#c.PairAge85#c.IsDZ=0), small
display "-2LL = " e(ll)*-2   // Print -2LL for model
estimates store FitZyg       // Save LL for LRT

/*
display "STATA Model 3c: Add Heterogeneous G and R matrices by Zygosity"
display "My best attempt following Example 6 on p. 498 of mixed manual, but it doesn't work"
mixed info c.time c.time#c.time c.PairAge85 c.time#c.PairAge85 c.time#c.time#c.PairAge85   ///
           c.IsDV c.time#c.IsDV c.time#c.time#c.IsDV                                 ///
           c.PairAge85#c.IsDV c.time#c.PairAge85#c.IsDV c.time#c.time#c.PairAge85#c.IsDV,  ///
        || PairID: ibn.zygosity ibn.zygosity#c.time, noconstant covariance(unstructured) /// Level 3
        || TwinID: ibn.zygosity ibn.zygosity#c.time, noconstant covariance(unstructured) /// Level 2
           reml dfmethod(satterthwaite) dftable(pvalue) nolog residuals(independent,by(zygosity)) // L1 R
display "-2LL = " e(ll)*-2    // Print -2LL for model
estimates store FitHet        // Save LL for LRT
lrtest FitHet FitZyg          // Test het variances by zygosity 
*/

 // Close external file of saved results    
 log close
